from __future__ import annotations
import sys
from enum import Enum
from functools import total_ordering


# https://docs.python.org/3/library/functools.html#functools.total_ordering
@total_ordering
class WarnLevel(Enum):
    NONE = 0  # Not a warning. Everything is ok.
    LOW = 1
    NORMAL = 2
    HIGH = 3

    def __eq__(self, other):
        if isinstance(other, WarnLevel):
            return self.value == other.value
        return NotImplemented

    def __lt__(self, other):
        if isinstance(other, WarnLevel):
            return self.value < other.value
        return NotImplemented


class WarnMessage:
    def __init__(self, level: WarnLevel, message: str = None):
        self.level = level
        self.message = message

    def is_warning(self) -> bool:
        return self.level > WarnLevel.NONE

    def print(self, default_message: str = 'Warning!') -> WarnMessage:
        """
        return: self
        """
        message = default_message if self.message is None else self.message

        if self.level == WarnLevel.NONE:
            pass
        elif self.level == WarnLevel.LOW:
            print(message)
        elif self.level == WarnLevel.NORMAL:
            print(message, file=sys.stderr)
        elif self.level == WarnLevel.HIGH:
            print(f'[CRITICAL] {message}', file=sys.stderr)

        return self
